package l16_streams;

import java.util.*;
import java.util.function.Predicate;
import java.util.stream.Stream;

import static java.util.stream.Collectors.*;

public class ProgrammersTest {

	record Employee(String firstName, String lastName, String gender, int age, double salary) {
		@Override
		public String toString() {
			return String.format("%s %s: %.2f", firstName, lastName, salary);
		}
	}

	public static void main(String[] args) {

		List<Employee> javaProgrammers = List.of(
			new Employee("Elmer", "Jacob", "male", 43, 2000),
			new Employee("Thomsen", "Brittany", "female", 23, 1500),
			new Employee("Floyd", "Donny", "male", 33, 1800),
			new Employee("Cindy", "Jonie", "female", 32, 1600),
			new Employee("Vere", "Hervey", "male", 22, 1200),
			new Employee("Maude", "Jaimie", "female", 27, 1900),
			new Employee("Shawn", "Randall", "male", 30, 2300),
			new Employee("Jayden", "Corrina", "female", 35, 1700),
			new Employee("Palmer", "Dene", "male", 33, 2000),
			new Employee("Addison", "Pam", "female", 34, 1300)
		);

		List<Employee> phpProgrammers = List.of(
			new Employee("Jarrod", "Pace", "male", 34, 1550),
			new Employee("Clarette", "Cicely", "female", 23, 1200),
			new Employee("Victor", "Channing", "male", 32, 1600),
			new Employee("Tori", "Sheryl", "female", 21, 1000),
			new Employee("Osborne", "Shad", "male", 32, 1100),
			new Employee("Rosalind", "Layla", "female", 25, 1300),
			new Employee("Fraser", "Hewie", "male", 36, 1100),
			new Employee("Quinn", "Tamara", "female", 21, 1000),
			new Employee("Alvin", "Lance", "male", 38, 1600),
			new Employee("Evonne", "Shari", "female", 40, 1800)
		);

		// Print PHP programmers that earn more than $1,400
		System.out.println("PHP programmers that earn more than $1,400:");
		phpProgrammers.stream()
			.filter((p) -> (p.salary() > 1400))
			.forEach(System.out::println);
		System.out.println();

		// Define some filters
		Predicate<Employee> ageFilter    = employee -> employee.age() > 25;
		Predicate<Employee> salaryFilter = employee -> employee.salary() > 1400;
		Predicate<Employee> genderFilter = employee -> employee.gender().equals("female");

		System.out.println("Female PHP programmers that earn more than $1,400 and are older than 25 years:");
		phpProgrammers.stream()
			.filter(ageFilter)
			.filter(salaryFilter)
			.filter(genderFilter)
			.forEach(System.out::println);
		System.out.println();

		// Reuse filters
		System.out.println("Female Java programmers older than 25 years:");
		javaProgrammers.stream()
			.filter(ageFilter)
			.filter(genderFilter)
			.forEach(System.out::println);
		System.out.println();

		// limit examples
		System.out.println("Print first 3 Java programmers:");
		javaProgrammers.stream()
			.limit(3)
			.forEach(System.out::println);
		System.out.println();

		System.out.println("Print first 3 female Java programmers:");
		javaProgrammers.stream()
			.filter(genderFilter)
			.limit(3)
			.forEach(System.out::println);
		System.out.println();

		// sorted, collect, limit, min, max examples
		System.out.println("Sort and print first 5 Java programmers by name:");
		List<Employee> sortedJavaProgrammers = javaProgrammers
			.stream()
			.sorted(Comparator.comparing(Employee::firstName))
			.limit(5)
			.toList();
		sortedJavaProgrammers.forEach(System.out::println);
		System.out.println();

		System.out.println("Sort and print Java programmers by salary:");
		sortedJavaProgrammers = javaProgrammers
			.stream()
			.sorted(Comparator.comparingDouble(Employee::salary))
			.toList();
		sortedJavaProgrammers.forEach(System.out::println);
		System.out.println();

		// min is faster than sorting and choosing the first
		System.out.println("Get the lowest Java programmer salary:");
		javaProgrammers
			.stream()
			.min(Comparator.comparingDouble(Employee::salary))
			.ifPresent(System.out::println);
		System.out.println();

		System.out.println("Get the highest Java programmer salary:");
		javaProgrammers
			.stream()
			.max(Comparator.comparingDouble(Employee::salary))
			.ifPresent(System.out::println);
		System.out.println();

		// map, collect examples
		System.out.println("Get PHP programmers first name to String:");
		String phpDevelopers = phpProgrammers
			.stream()
			.map(Employee::firstName)
			.collect(joining("; "));
		System.out.println(phpDevelopers);
		System.out.println();

		System.out.println("Get Java programmers first name to Set:");
		Set<String> javaDevFirstName = javaProgrammers
			.stream()
			.map(Employee::firstName)
			.collect(toSet());
		System.out.println(javaDevFirstName);
		System.out.println();

		System.out.println("Get Java programmers last name to TreeSet:");
		TreeSet<String> javaDevLastName = javaProgrammers
			.stream()
			.map(Employee::lastName)
			.collect(toCollection(TreeSet::new));
		System.out.println(javaDevLastName);
		System.out.println();

		// parallel stream, sum examples
		System.out.printf("Parallel version on %s-core machine%n", Runtime.getRuntime().availableProcessors());
		System.out.println("Calculate total money spent for paying Java programmers:");
		double totalSalary = javaProgrammers
			.parallelStream()
			.mapToDouble(Employee::salary)
			.sum();
		System.out.printf("Money spent for paying Java programmers: $%.2f%n", totalSalary);
		System.out.println();

		//Get count, min, max, sum, and average for numbers
		DoubleSummaryStatistics stats =
			Stream.of(javaProgrammers, phpProgrammers)
			.flatMap(List::stream)
			.mapToDouble(Employee::salary)
			.summaryStatistics();

		System.out.println("Java/PHP programmers salary statistics");
		System.out.println(stats);
	}
}

